#! /usr/bin/perl

# Photo album.  
# Displays photos.  :-)  For quick setup info, see the "CONFIGURATION"
# section below.
#
# Copyright 1999-2001 by Robert Kesterson, robertk@robertk.com
#
#    LICENSE
#
# You may use, copy, or redistribute this file as you wish, so long as 
# the above copyright notice and the copyright notice in the generated 
# HTML comments remains intact, and that if you modify the program,
# you must note that it has been modified from the original version.
#
#
#    USAGE 
#
# Images and captions are pulled from files in a directory that
# bears the name of the album.  Header and footer are pulled
# from separate files in that directory (see below).
#
# To create a new album, simply create a directory under your "albums"
# directory, and in that directory, place all your photos.  For every photo,
# create a text file which contains the caption (you can add HTML tags if you 
# like), and name it the same thing as the photo, but with a ".caption"
# extension.  If you want a thumbnail of each photo (recommended), make it much 
# smaller (no more than 100 pixels tall), and name it with the extension 
# ".thumb.jpg"   (If you're on a Linux system, you can create thumbnails for
# a whole directory of jpgs quite easily if you have the "convert" tool 
# installed.  Just type the following *all as one line* in your bash shell:
#         for file in *.jpg; do echo $file; 
#         convert -geometry 100x100 $file ${file%.jpg}.thumb.jpg; done
# That will create thumbnails for all the jpgs in the current directory.
# Handy, eh?)
#
# Then for your header and footer, just create them as "header.html" and 
# "footer.html".  (They will be passed through the script unaltered, with the 
# photos inserted between them.)
#
# For example, a directory might contain the following:
#     header.html
#     footer.html
#     myphoto.jpg
#     myphoto.caption
#     myphoto.thumb.jpg
#     another.jpg
#     another.caption
#     another.thumb.jpg
# and so forth.
#
#
# You do NOT need to modify this script to add new photos!  Just create them
# in the format noted above and they will automatically be added!
#
# HISTORY
#
# Version 1.1, June 20, 2001
# Made it simpler to set up by adding an url dir for the images, so your 
# image path for the script and the url path for the web page aren't as 
# tightly coupled.  Also removed all unix-specific stuff so this should work
# anyplace you can run Perl...
#
# Version 1.2, August 10,2001
# Broke up the index so that it can span multiple pages.  Makes index
# loading faster.  Displays a page counter so you know where you are in
# the index.  Number of images on each index page is configurable.
#

# ********************************************************************************
#             CONFIGURATION SECTION
# ********************************************************************************
             
#
# Here are some global config variables you might need to change
#

# $albumdir = the root directory in which to look for photo album directories.  
#             All your albums should be subdirectories of this directory.  
#             Note that you must use an absolute SYSTEM path.
#             (That is, if your HTML is in your "/home/jdoe/public_html"
#             directory, and your albums are in the "albums" directory under
#             your HTML directory, your $albumdir must be set to 
#             "/home/jdoe/public_html/albums"  Make sense?)
# $urldir = the location of the $albumdir *for the browser*.  So again going on
#           the above example (with the html being in /home/jdoe/public_html
#	    and albums in a directory under that), the urldir should be set to
#           "/albums" -- ie, the path relative to the top HTML directory.
# $scriptlocation = the relative path back to this script (i.e, "/cgi-bin/photoalbum.cgi");
# $numbthumbs = how many thumbnails to display on EACH side of the page.
# $numcols = how many "columns" of pictures to show per page.  
# $numrows = how many "rows" of pictures to show per page
# $prevbutton = image to use for previous page button
# $nextbutton = image to use for next page button
# $indexbutton = image to use for index button
# $thumbsperrow = number of thumbnails per row /column on the index pages.

$albumdir = "/home/joeuser/public_html/albums";
$urldir = "/albums";
$scriptlocation = "/cgi-bin/photoalbum.cgi";
$prevbutton = "/images/photoprev.gif";
$nextbutton = "/images/photonext.gif";
$indexbutton = "/images/photoindex.gif";
$numthumbs = 3;
$numcols = 3;
$numrows = 3;
$thumbsperrow = 5;


# ********************************************************************************
# Changes below this line will affect (and possibly break) the script!
# ********************************************************************************
             
# set the environment and include some common functions
require ("cgi_lib.pl");

# which album to display and which picture to start display from.  
$Album = "";
$StartPic = "";
$StartIndex = 0;
$ShowIndex = "No";
$IndexFirst = 0;

# info from ScanDirectory is returned in these
# %PicList  is a hash of filenames to index numbers
# @PicInfo is a sorted array of picture info structures.

$nextthumbs = 0;
                  
sub GetEnvironment()
{
   # Read the starting picture name, if any, and start the album display at that point. 
   &ReadParse(*pargs);
  
  $StartPic = $pargs{"StartPic"};
  $Album = $pargs{"Album"};
  $ShowIndex = $pargs{"ShowIndex"};
  $numcols = $pargs{"NC"};
  $IndexFirst = $pargs{"First"};
  if($numcols < 1)
  {
	 $numcols = 1;
  }
  $numrows = $pargs{"NR"};
  if($numrows < 1)
  {
	  $numrows = 1;
  }
}

sub GetPictureInfo($)
{
   local %picinf;
   local $basename, $extn;

   $basename = substr($_[0], 0, length($_[0])-4);
   $picinf{'basename'} = $basename;
   $picinf{'photo'} = $_[0];
   $picinf{'caption'} = "$basename.caption";
   $picinf{'thumbnail'} = "$basename.thumb.jpg";
   return %picinf;
}

sub ScanDirectory()
{
   # OK.  First we scan the directory to get a list of file names.  Then we sort them
   # and save the sorted results to an associative array (so we can look up the start pic by name).

   $idex = 0;
   opendir DIR, "$albumdir/$Album" or die "Can't open album directory";
   while($file = readdir(DIR))
   {
        chomp($file);
        if (length($file) < 4)
        {
           $file = "";
        }
		if(length($file) > 10) 
		{
		   $test = substr($file, length($file)-10, 10);
		   if($test eq ".thumb.jpg")
		   {
			  $file = "";
		   }
		}
        $test = substr($file, length($file) - 4, 4);
        if($test ne ".jpg")
        {
           $file = "";
        }
		if($file gt "")
		{
			 $PicList{$file} = $idex;
			 %picinf = GetPictureInfo($file);      
			 $PicInfo[$idex] = { basename => $picinf{'basename'}, photo => $picinf{'photo'}, caption => $picinf{'caption'}, thumbnail => $picinf{'thumbnail'}, };
			 $idex++;
		}
   }
   closedir(DIR);
}

sub ShowPriorThumbnails()
{
   $thumbstart = $StartIndex - $numthumbs;
   print "<td rowspan=$numrows>\n";
   print "<center><font size=-1 color=\"FF0000\"><i>Previous Pages</i></font></center><P>\n";
   # now loop through the picture array, inserting the required number of 
   # thumbnails, and linking them to a new album page.
   for($picdex=0; $picdex < $numthumbs; $picdex++)
   {
      $target = $thumbstart + $picdex;
	  $href = $PicInfo[$target];
	  if(($target >= 0) && ($target <= $StartIndex)) 
	  {
	  	   print "<a href=\"$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$href->{'photo'}\"><img border=0 src=\"$urldir/$Album/$href->{'thumbnail'}\"></a><p>\n";
	  }
   }      
   print "</td>\n";
}

sub ShowNextThumbnails()
{
   $thumbstart = $StartIndex + ($numrows * $numcols);
  print "<td rowspan=$numrows>\n";
  print "<center><font size=-1 color=\"FF0000\"><i>Upcoming Pages</i></font></center><P>\n";
  # now loop through the picture array, inserting the required number of 
  # thumbnails, and linking them to a new album page.
	for($picdex=0; $picdex < $numthumbs; $picdex++)
	{
		 $target = $thumbstart + $picdex;
		 $href = $PicInfo[$target];
		 if(($target >= 0) && ($target < @PicInfo))
		 {
			print "<a href=\"$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$href->{'photo'}\"><img border=0 src=\"$urldir/$Album/$href->{'thumbnail'}\"></a><p>\n";
		 }
   }      
  print "</td>\n";
}

sub PassThru($) 
{
   $fhopen = open (FH, "$_[0]");
   if ($fhopen != 0)
   {
       while(<FH>)
       {
          print "$_";
       }
       close(FH);
   }
}

sub ShowPictures()
{
	$picdex = $StartIndex;
	for($row = 0; $row < $numrows; $row++)
	{
	  $picdex = $StartIndex + ($numcols * $row);
	  if($row > 0)
	  {
		 # don't open first row -- main routine already opens it.
		 print "<tr valign=top>\n";
	  }
	  for($col = 0; $col < $numcols; $col++)
	  {
		 $href = $PicInfo[$picdex];
		 if($href->{'photo'} gt "")
		 {
			print "<td align=center valign=center>\n";         
			print "<center>\n";         
			print "<img src=\"$urldir/$Album/$href->{'photo'}\"><br>\n";
			print "<font size=-1>\n";
            # read the caption file
            PassThru("$albumdir/$Album/$href->{'caption'}");
			print "</font></p>\n";
			print "</center>\n";
			print "</td>\n";
		 }
		 $picdex++;
	  }
	  if($row < $numrow -1)
	  {
		 # don't close last row, main script will close it.
		 print "</tr>\n";
	  }
	  else
	  {
		 # but if closing this row, show the thumbs for the next page.
		 if($nextthumbs == 0)
		 {
			$nextthumbs = 1;
			ShowNextThumbnails();
		 }
	  }
   }      
}

sub ShowIndex()
{                     
   local $mynumrows, $prevpage, $nextpage, $picsused;                              
   local $curpage, $totpages;
   # print the HTTP header
   print &PrintHeader;

   # Now read the album and find a list of pictures we're concerned about.                        
   # ScanDirectory() creates an associative array ($PicList) with the picture names
   # and their index numbers, and a regular array ($PicInfo) with the picture info in
   # sorted order by index number.
   ScanDirectory();  
   $picsused = $thumbsperrow * $thumbsperrow - 1;
   $curpage = $IndexFirst / $picsused + 0.5 + 1; # 1 is for 1-relative 
   $totpages = (@PicInfo / $picsused) + 0.5;     # 0.5 is for rounding
   
   #
   # First print the header, unaltered.
   #

   PassThru("$albumdir/$Album/header.html");

   print <<EOS;
<!--*****************************************************************************-->
<!--HTML Photo Album by Robert Kesterson, robertk\@robertk.com-->
<!--*****************************************************************************-->
<center><br>
<table border=1>
EOS
   printf("<tr><td colspan=%d align=right>Page %d of
%d</td></tr>", $thumbsperrow, $curpage,$totpages);
   print "<tr valign=top>";

   # now just show all the thumbnails.
   $maxperrow = $thumbsperrow;
   $picsrow = 0;  
   $mynumrows = 0; 
   for($counter = 0; $counter < @PicInfo; $counter++)
   {
	if(($counter >= $IndexFirst) && ($mynumrows < $thumbsperrow))
	{
	  $href = $PicInfo[$counter];
	  print "<td align=center valign=center><a
href=\"$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$href->{'photo'}\">
<img border=0 src=\"$urldir/$Album/$href->{'thumbnail'}\"</a></td>\n";
	  $picsrow++;
	  if($picsrow == $maxperrow)
	  {
		 $picsrow = 0;
		 $mynumrows++;
		 print "</tr>\n";
	  }
	}
   }
   $prevpage = 0;
   $nextpage = 0;
   if($IndexFirst - $thumbsperrow * $thumbsperrow > 0)
   {
	$prevpage = $IndexFirst - $thumbsperrow * $thumbsperrow;
   }
   $picsused = $IndexFirst + $mynumrows * $thumbsperrow + $picsrow;
   if($picsused < @PicInfo)
   {
      $nextpage = $picsused - 1;
   }

	$firstpic = $PicInfo[0];
	$lastpic = $PicInfo[@PicInfo-1];

   print <<EOS;
</tr>
<tr>
<td colspan=$maxperrow align=left>
<table width=100% border=0 cellspacing=0 cellpadding=0>
<tr>
<td width=33% align=left>
EOS
   if($IndexFirst > 0)
   {
   print <<EOS;
<a
href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes&First=$prevpage"><img
border=0 src="$prevbutton"></a>
	  <br><a
href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes&First=$prevpage"><font
size=-2>Previous Page</font></a>
EOS
   }
   else
   {
      print "&nbsp;";
   }
   print "</td>";
   print <<EOS;
<td align=center><a
href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes"><img border=0 src="$indexbutton"></a>
	  <br><a href="$scriptlocation?Album=$Album&ShowIndex=Yes"><font size=-2>Index</font></a></td>
<td width=33% align=right>
EOS
   if($nextpage > 0)
   {
      print <<EOS;
<a
href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes&First=$nextpage"><img
border=0 src="$nextbutton"></a>
	  <br><a
href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes&First=$nextpage"><font
size=-2>Next Page</font></a>
EOS
   }
   else
   {
      print "&nbsp;";
   }
   print "</td></tr></table></td></tr></table></center>";

   PassThru("$albumdir/$Album/footer.html");
}

sub ShowAlbum()
{                                                                                                                
   local $ShowPrev, $ShowNext;
   # print the HTTP header
   print &PrintHeader;

   # Now read the album and find a list of pictures we're concerned about.                        
   # ScanDirectory() creates an associative array ($PicList) with the picture names
   # and their index numbers, and a regular array ($PicInfo) with the picture info in
   # sorted order by index number.
   ScanDirectory();  
   
   #
   # First print the header, unaltered.
   #

   PassThru("$albumdir/$Album/header.html");

   
   # now find the first picture to show and show the pictures from there.  
   # Show a column of thumbnails for next and previous on either side.
   $StartIndex = $PicList{$StartPic};
   
   print <<EOS;
<!--*****************************************************************************-->
<!--HTML Photo Album by Robert Kesterson, robertk\@robertk.com-->
<!--*****************************************************************************-->
<center><br>
<table border=1 bgcolor="#EEEECC">
<tr valign=top>
EOS

   $nextthumbs = 0;
   ShowPriorThumbnails();
   ShowPictures();   
   if($nextthumbs == 0)
   {
	  ShowNextThumbnails();   
	}
    
	$colspan = $numcols + 2;
	$firstpic = $PicInfo[$StartIndex];
   if($StartIndex > ($numcols * $numrows))
   {                                    
	  $firstpic = $PicInfo[$StartIndex - ($numcols * $numrows)];
   }
   else
   {
	  $firstpic = $PicInfo[0];
   }

   $ShowPrev = 0;
   if($StartIndex > 0)
   {
      $ShowPrev = 1;
   }   
   $lastpic = $PicInfo[$StartIndex + ($numrows * $numcols)];
   $ShowNext = 0;
   if($StartIndex + $numrows * $numcols < @PicInfo)
   {
	$ShowNext = 1;
   }
   print <<EOS;
</tr>
<tr>
<td colspan=$colspan align=left>
<table width=100% border=0 cellspacing=0 cellpadding=0>
<tr>
<td width=33% align=left>
EOS
   if($ShowPrev)
   {
      print <<EOS;
<a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$firstpic->{'photo'}"><img border=0 src="$prevbutton"></a>
	  <br><a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$firstpic->{'photo'}"><font size=-2>Previous Page</font></a>
EOS
   }
   else
   {
      print "&nbsp;";
   }
   print "</td>";
   print <<EOS;
<td align=center><a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes"><img border=0 src="$indexbutton"></a>
	  <br><a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&ShowIndex=Yes"><font size=-2>Index</font></a></td>
<td width=33% align=right>
EOS
   if($ShowNext)
   {
      print <<EOS;
<a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$lastpic->{'photo'}"><img border=0 src="$nextbutton"></a>
	  <br><a href="$scriptlocation?Album=$Album&NC=$numcols&NR=$numrows&StartPic=$lastpic->{'photo'}"><font size=-2>Next Page</font></a>
EOS
   }
   else
   {
      print "&nbsp;";
   }
   print <<EOS;
</td>
</tr>
</table>
</td>
</tr>
</table>
</center>
EOS

   PassThru("$albumdir/$Album/footer.html");
}

# Main Program                            

&GetEnvironment();        
if($ShowIndex eq "Yes")
{
   &ShowIndex();
}
else
{   
   &ShowAlbum();
}   
